# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

import os
import json
import flask


APP = flask.Flask(__name__)
APP.config.from_object('{}.defaults'.format(__name__))
if 'FLASK_SETTINGS' in os.environ:
    APP.config.from_envvar('FLASK_SETTINGS')

#
# Database
#


@APP.before_request
def db_connect():
    from .database import Database
    flask.g.db = Database(
        APP.name, APP.config['SQLALCHEMY_DATABASE_URI']
    ).get_session()


@APP.teardown_appcontext
def db_disconnect(exception=None):
    if hasattr(flask.g, 'db'):
        flask.g.db.remove()


#
# Views
#

# Imported here to avoid circular imports.
from .auth import api_token_required  # flake8:noqa
from .karma import KarmaManager, NoSuchFASUser, VoteAlreadyExists  # flake8:noqa


@APP.route('/user/<username>', methods=['GET'])
def view_user_get(username):
    # Get the user's stats
    karma_manager = KarmaManager(flask.g.db)
    stats = karma_manager.stats(username)
    stats.update(dict(username=username))
    return flask.jsonify(stats)


@APP.route('/user/<username>', methods=['POST'])
@api_token_required()
def view_user_post(username):
    sender = flask.request.form.get('sender', None)
    if not sender:
        return flask.abort(400)
    if sender == username:
        return "You may not modify your own karma.", 403
    karma_manager = KarmaManager(flask.g.db)
    # Increment / Decrement
    increment = ('decrement' not in flask.request.form or
                 not bool(flask.request.form['decrement']))
    # Extra message data
    extra = flask.request.form.get('extra', '')
    try:
        extra = json.loads(extra)
    except ValueError:
        extra = {}
    stats = dict(username=username)
    # Now change the user's karma
    try:
        karma_manager.change(sender, username, increment, extra)
    except NoSuchFASUser as error:
        return str(error), 404
    except VoteAlreadyExists:
        stats['existing'] = True
    stats.update(karma_manager.stats(username))
    return flask.jsonify(stats)


# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
