# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

from alembic import command as alembic_command
from alembic.config import Config as AlembicConfig
from alembic.migration import MigrationContext
from alembic.script import ScriptDirectory
from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import scoped_session, sessionmaker


class Database(object):

    def __init__(self, module, db_url):
        self.db_url = db_url
        self._engine = None
        self.alembic_cfg = AlembicConfig()
        self.alembic_cfg.set_main_option(
            "script_location", "%s:migrations" % module)

    @property
    def engine(self):
        if self._engine is None:
            self._engine = create_engine(self.db_url, convert_unicode=True)
            if self._engine.name == "sqlite":
                self._engine.execute("PRAGMA foreign_keys=ON")
        return self._engine

    def get_session(self):
        session = scoped_session(sessionmaker(
            autocommit=False, autoflush=False, bind=self.engine))
        from .models import Base
        Base.query = session.query_property()
        return session

    def get_revision(self):
        # Get the current database revision
        alembic_context = MigrationContext.configure(self.engine)
        return alembic_context.get_current_revision()

    def get_head_revision(self):
        # Get the most up-to-date database revision
        script_dir = ScriptDirectory.from_config(self.alembic_cfg)
        return script_dir.get_current_head()

    def create(self):
        # import all modules here that might define models so that
        # they will be registered properly on the metadata.  Otherwise
        # you will have to import them first before calling init_db()
        from .models import Base
        Base.metadata.create_all(bind=self.engine)
        alembic_command.stamp(self.alembic_cfg, "head")

    def upgrade(self):
        alembic_command.upgrade(self.alembic_cfg, "head")


# Migration helpers

def is_sqlite(bind):
    return bind.dialect.name == 'sqlite'


def exists_in_db(bind, tablename, columnname=None):
    md = MetaData()
    md.reflect(bind=bind)
    if columnname is None:
        return tablename in md.tables
    else:
        return (
            tablename in md.tables and
            columnname in [c.name for c in md.tables[tablename].columns]
        )


# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
