# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

import fedmsg
import socket

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.expression import func

from .lib import make_fas_client, get_current_release, cache
from .models import Vote

import logging
logger = logging.getLogger(__name__)


class NoSuchFASUser(ValueError):
    pass


class VoteAlreadyExists(ValueError):
    pass


class KarmaManager:

    def __init__(self, db):
        self.db = db
        self.fasclient = make_fas_client()

    def _username_exists(self, username):
        cache_key = "username_exists:{}".format(username)
        result = cache.get(cache_key)
        if result is None:
            request = self.fasclient.send_request(
                'json/person_by_username',
                auth=True,
                req_params={'username': username})
            result = bool(request['success'])
            if result:
                # Cache for a day, usernames are unlikely to go away.
                # Don't cache if the username does not exist (yet).
                cache.set(cache_key, result, timeout=3600 * 24)
        return result

    def change(self, agent, recipient, increment=True, extra=None):
        # Send the karma change to fedmsg.
        # Store the change in the database.

        # Check that the sender and the recipient are FAS users.
        for username in (agent, recipient):
            if not self._username_exists(username):
                logger.info(
                    "Saw %s from %s, but %s not in FAS",
                    recipient, agent, username)
                raise NoSuchFASUser("Couldn't find %s in FAS." % username)

        release = get_current_release()
        vote = 1 if increment else -1

        # Check our karma db to make sure this hasn't already been done.
        try:
            existing = self.db.query(Vote).filter_by(
                release=release, from_user=agent, to_user=recipient).one()
        except NoResultFound:
            logger.debug(
                "Adding a vote to the DB: %s -> %s (%s)",
                recipient, agent, release)
            self.db.add(Vote(
                release=release,
                from_user=agent,
                to_user=recipient,
                value=vote,
            ))
        else:
            # Update the existing vote if necessary.
            if existing.value == vote:
                raise VoteAlreadyExists(
                    "{} has already given {} karma to {}".format(
                        agent, vote, recipient))
            existing.value = vote

        self.db.commit()

        base_query = self.db.query(
            func.sum(Vote.value)).filter_by(to_user=recipient)
        total_all_time = base_query.scalar()
        total_this_release = base_query.filter_by(release=release).scalar()

        logger.debug("Sending a message to fedmsg")
        hostname = socket.gethostname().split('.')[0]
        if extra and isinstance(extra, dict):
            msg = extra.copy()
        else:
            msg = {}
        msg.update({
            'agent': agent,
            'recipient': recipient,
            'total': total_all_time,  # The badge rules use this value
            'total_this_release': total_this_release,
            'vote': vote,
            'release': release,
        })
        fedmsg.publish(
            name="plus-plus-service.%s" % hostname,
            modname="plus-plus-service", topic="karma",
            msg=msg,
        )

    def stats(self, username):
        release = get_current_release()
        base_query = self.db.query(Vote).filter_by(to_user=username)

        inc = base_query.filter_by(release=release, value=1).count()
        dec = base_query.filter_by(release=release, value=-1).count()
        current = inc - dec

        alltime_inc = base_query.filter_by(value=1).count()
        alltime_dec = base_query.filter_by(value=-1).count()
        total = alltime_inc - alltime_dec

        return dict(
            release=release,
            increments=inc,
            decrements=dec,
            current=current,
            total=total,
        )


# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
