# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

import argparse
import os
import os.path
import sys

from . import APP
from .database import Database


class Command:
    """Base command class"""

    name = None
    _help_text = None
    _parser = None
    _arguments = []

    def make_parser(self, subparser):
        self._parser = subparser.add_parser(self.name, help=self._help_text)
        self._parser.set_defaults(func=self.run)
        for arg in self._arguments:
            self._parser.add_argument(arg)

    def run(self, args):
        raise NotImplementedError


class SyncDB(Command):
    name = "syncdb"

    def run(self, args):
        db = Database(
            APP.name, APP.config["SQLALCHEMY_DATABASE_URI"])
        current_rev = db.get_revision()
        # If the database is empty, it should be created ; otherwise it should
        # be upgraded.
        if current_rev is None:
            db.create()
            print("Database created.")
        elif current_rev == db.get_head_revision():
            print("Database already up-to-date.")
        else:
            db.upgrade()
            print("Database upgraded.")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--config", help="Load this configuration file")
    subparsers = parser.add_subparsers(help='sub-command help')
    for action_class in Command.__subclasses__():
        action_class().make_parser(subparsers)
    args = parser.parse_args()
    if args.config is not None:
        args.config = os.path.abspath(args.config)
        if os.path.exists(args.config):
            APP.config.from_pyfile(args.config)
        else:
            parser.error("can't find the config file: %s" % args.config)
    if not hasattr(args, "func"):
        parser.print_help()
        return
    sys.exit(args.func(args))
