# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

from mock import Mock, patch
from .utils import TestCase
from ..models import Vote
from ..karma import KarmaManager


class KarmaTestCase(TestCase):

    def setUp(self):
        super().setUp()
        self.karma_manager = KarmaManager(self.db)
        self.karma_manager.fasclient = Mock()
        self.karma_manager.fasclient.send_request.return_value = dict(
            success=True)
        self.fedmsg_patcher = patch('fedmsg.publish')
        self.fedmsg = self.fedmsg_patcher.start()

    def tearDown(self):
        super().tearDown()
        self.fedmsg_patcher.stop()

    def test_stats(self):
        # 1 vote on release-1, 3 votes on release-2 but one of them is
        # negative.
        self.db.add(Vote(
            release="release-1",
            from_user="from-1",
            to_user="target",
            value=1))
        self.db.add(Vote(
            release="release-2",
            from_user="from-1",
            to_user="target",
            value=1))
        self.db.add(Vote(
            release="release-2",
            from_user="from-2",
            to_user="target",
            value=1))
        self.db.add(Vote(
            release="release-2",
            from_user="from-3",
            to_user="target",
            value=-1))
        self.db.commit()
        with patch('plus_plus_service.karma.get_current_release') as gcr:
            gcr.return_value = "release-2"
            result = self.karma_manager.stats("target")
        self.assertEqual(result, {
            'release': 'release-2',
            'current': 1,
            'increments': 2,
            'decrements': 1,
            'total': 2,
        })

    def test_change(self):
        with patch('plus_plus_service.karma.get_current_release') as gcr:
            gcr.return_value = "release-1"
            self.karma_manager.change("source", "target")
        self.assertEqual(self.fedmsg.call_count, 1)
        call_kw = self.fedmsg.call_args_list[0][1]
        self.assertEqual(call_kw["msg"], {
            'agent': "source",
            'recipient': "target",
            'total': 1,
            'total_this_release': 1,
            'vote': 1,
            'release': "release-1",
        })
        self.assertEqual(self.db.query(Vote).count(), 1)
        vote = self.db.query(Vote).one()
        self.assertEqual(vote.release, "release-1")
        self.assertEqual(vote.from_user, "source")
        self.assertEqual(vote.to_user, "target")
        self.assertEqual(vote.value, 1)
        # source and target must have been checked in FAS
        send_request_mock = self.karma_manager.fasclient.send_request
        self.assertEqual(send_request_mock.call_count, 2)
        self.assertEqual(
            send_request_mock.call_args_list[0][1]["req_params"],
            {'username': 'source'})
        self.assertEqual(
            send_request_mock.call_args_list[1][1]["req_params"],
            {'username': 'target'})

    def test_change_extra(self):
        with patch('plus_plus_service.karma.get_current_release') as gcr:
            gcr.return_value = "release-1"
            self.karma_manager.change(
                "source", "target", extra={"extra": "value"})
        self.assertEqual(self.fedmsg.call_count, 1)
        call_kw = self.fedmsg.call_args_list[0][1]
        self.assertTrue("extra" in call_kw["msg"])
        self.assertEqual(call_kw["msg"]["extra"], "value")

    def test_change_extra_no_override(self):
        with patch('plus_plus_service.karma.get_current_release') as gcr:
            gcr.return_value = "release-1"
            self.karma_manager.change(
                "source", "target", extra={"vote": 1000})
        self.assertEqual(self.fedmsg.call_count, 1)
        call_kw = self.fedmsg.call_args_list[0][1]
        self.assertEqual(call_kw["msg"]["vote"], 1)

    def test_change_extra_invalid(self):
        with patch('plus_plus_service.karma.get_current_release') as gcr:
            gcr.return_value = "release-1"
            self.karma_manager.change(
                "source", "target", extra="dummy-value")
        self.assertEqual(self.fedmsg.call_count, 1)
        call_kw = self.fedmsg.call_args_list[0][1]
        self.assertEqual(
            sorted(call_kw["msg"].keys()),
            ['agent', 'recipient', 'release', 'total',
             'total_this_release', 'vote']
        )
