# This file is part of Plus Plus Service.
# Copyright (C) 2016 The Fedora Project
#
# Plus Plus Service is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Plus Plus Service is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Plus Plus Service.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:  Aurelien Bompard <abompard@fedoraproject.org>
#

import json

from mock import call, Mock, patch
from .utils import TestCase
from .. import APP
from ..models import Vote
from ..karma import VoteAlreadyExists


AUTH_HEADER = {
    'Authorization': 'token {}'.format(APP.config['PLUS_PLUS_TOKEN'])
}


class ViewTestCase(TestCase):

    def setUp(self):
        super().setUp()
        self.karma_manager = Mock()
        self.kmgr_patcher = patch("plus_plus_service.KarmaManager")
        kmgr_class = self.kmgr_patcher.start()
        kmgr_class.return_value = self.karma_manager

    def tearDown(self):
        super().tearDown()
        self.kmgr_patcher.stop()

    def test_get(self):
        dummy_values = dict(
            release='release-1',
            current=2,
            increments=3,
            decrements=4,
            total=5,
        )
        self.karma_manager.stats.return_value = dummy_values
        response = self.client.get('/user/target')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.karma_manager.stats.call_count, 1)
        result = json.loads(response.data.decode('ascii'))
        expected_values = dummy_values.copy()
        expected_values["username"] = "target"
        self.assertEqual(result, expected_values)

    def test_post(self):
        self.karma_manager.stats.return_value = {}
        with APP.app_context():
            response = self.client.post('/user/target',
                                        data=dict(sender='source'),
                                        headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.karma_manager.change.call_count, 1)
        self.assertEqual(
            self.karma_manager.change.call_args,
            call("source", "target", True, {}))
        result = json.loads(response.data.decode('ascii'))
        self.assertEqual(result, dict(username="target"))

    def test_return_403_when_sender_and_username_are_same(self):
        with APP.app_context():
            response = self.client.post('/user/target',
                                        data=dict(sender='target'),
                                        headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 403)

    def test_return_401_when_not_auth_header(self):
        self.karma_manager.stats.return_value = {}
        with APP.app_context():
            response = self.client.post('/user/target',
                                        data=dict(sender='source'))
        self.assertEqual(response.status_code, 401)

    def test_return_400_when_no_sender(self):
        with APP.app_context():
            response = self.client.post('/user/notarealfasuser',
                                        headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 400)

    def test_post_decrement(self):
        self.karma_manager.stats.return_value = {}
        with APP.app_context():
            response = self.client.post(
                '/user/target',
                data=dict(sender='source', decrement='1'),
                headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.karma_manager.change.call_count, 1)
        self.assertEqual(
            self.karma_manager.change.call_args,
            call("source", "target", False, {}))

    def test_post_decrement_empty(self):
        self.karma_manager.stats.return_value = {}
        with APP.app_context():
            response = self.client.post(
                '/user/target',
                data=dict(sender='source', decrement=''),
                headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.karma_manager.change.call_count, 1)
        self.assertEqual(
            self.karma_manager.change.call_args,
            call("source", "target", True, {}))

    def test_post_existing(self):
        self.db.add(Vote(
            release='release-1',
            from_user="source",
            to_user="target",
            value=1))
        self.db.commit()
        self.karma_manager.stats.return_value = {}
        self.karma_manager.change.side_effect = VoteAlreadyExists
        with APP.app_context():
            response = self.client.post('/user/target',
                                        data=dict(sender='source'),
                                        headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 200)
        result = json.loads(response.data.decode('ascii'))
        self.assertEqual(result, dict(
            username="target", existing=True))
        self.assertEqual(self.db.query(Vote).count(), 1)
        vote = self.db.query(Vote).one()
        self.assertEqual(vote.value, 1)

    def test_post_extra(self):
        self.karma_manager.stats.return_value = {}
        with APP.app_context():
            response = self.client.post(
                '/user/target',
                data=dict(sender='source', extra='{"extra": "value"}'),
                headers=AUTH_HEADER)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.karma_manager.change.call_count, 1)
        self.assertEqual(
            self.karma_manager.change.call_args,
            call("source", "target", True, {"extra": "value"}))
